#ifdef _MSC_VER
#include "ms_stdint.h"
#else
#include <stdint.h>
#endif

/********************************************************/
/* RDM Parameter ID's (Slots 21-22)	 Acc to RDM ESTA1.2 */
/********************************************************/
/* Category - Network Management   */
#define DISC_UNIQUE_BRANCH                           0x0001
#define DISC_MUTE                                    0x0002
#define DISC_UN_MUTE                                 0x0003
#define PROXIED_DEVICES                              0x0010
#define PROXIED_DEVICE_COUNT                         0x0011
#define COMMS_STATUS                                 0x0015

/* Category - Status Collection    */			       
#define QUEUED_MESSAGE                               0x0020 /* See Table A-4                                              */
#define STATUS_MESSAGES                              0x0030 /* See Table A-4                                              */
#define STATUS_ID_DESCRIPTION                        0x0031
#define CLEAR_STATUS_ID                              0x0032
#define SUB_DEVICE_STATUS_REPORT_THRESHOLD           0x0033 /* See Table A-4                                              */

/* Category - RDM Information      */			       
#define SUPPORTED_PARAMETERS                         0x0050 /* Support required only if supporting Parameters beyond the minimum required set.*/
#define PARAMETER_DESCRIPTION                        0x0051 /* Support required for Manufacturer-Specific PIDs exposed in SUPPORTED_PARAMETERS message */

/* Category - Product Information  */			      
#define DEVICE_INFO                                  0x0060
#define PRODUCT_DETAIL_ID_LIST                       0x0070
#define DEVICE_MODEL_DESCRIPTION                     0x0080
#define MANUFACTURER_LABEL                           0x0081
#define DEVICE_LABEL                                 0x0082
#define FACTORY_DEFAULTS                             0x0090
#define LANGUAGE_CAPABILITIES                        0x00A0
#define LANGUAGE                                     0x00B0
#define SOFTWARE_VERSION_LABEL                       0x00C0
#define BOOT_VERSION_SOFTWARE_ID                     0x00C1
#define BOOT_SOFTWARE_VERSION_LABEL                  0x00C2

/* Category - DMX512 Setup         */
#define DMX_PERSONALITY                              0x00E0
#define DMX_PERSONALITY_DESCRIPTION                  0x00E1
#define DMX_START_ADDRESS                            0x00F0 /* Support required if device uses a DMX512 Slot.             */
#define SLOT_INFO                                    0x0120
#define SLOT_DESCRIPTION                             0x0121
#define DEFAULT_SLOT_VALUE                           0x0122
#define DMX_BLOCK_ADDRESS                          0x0140 /* Defined in ANSI E1.37-1 document                           */
#define DMX_FAIL_MODE                              0x0141 /* Defined in ANSI E1.37-1 document                           */
#define DMX_STARTUP_MODE                           0x0142 /* Defined in ANSI E1.37-1 document                           */

/* Category - Sensors              */			       
#define SENSOR_DEFINITION                            0x0200
#define SENSOR_VALUE                                 0x0201
#define RECORD_SENSORS                               0x0202

/* Category - Dimmer Settings      */
#define DIMMER_INFO                                0x0340
#define MINIMUM_LEVEL                              0x0341
#define MAXIMUM_LEVEL                              0x0342
#define CURVE                                      0x0343
#define CURVE_DESCRIPTION                          0x0344 /* Support required if CURVE is supported                     */
#define OUTPUT_RESPONSE_TIME                       0x0345
#define OUTPUT_RESPONSE_TIME_DESCRIPTION           0x0346 /* Support required if OUTPUT_RESPONSE_TIME is supported      */
#define MODULATION_FREQUENCY                       0x0347
#define MODULATION_FREQUENCY_DESCRIPTION           0x0348 /* Support required if MODULATION_FREQUENCY is supported      */   

/* Category - Power/Lamp Settings  */			      
#define DEVICE_HOURS                                 0x0400
#define LAMP_HOURS                                   0x0401
#define LAMP_STRIKES                                 0x0402
#define LAMP_STATE                                   0x0403 /* See Table A-8                                              */
#define LAMP_ON_MODE                                 0x0404 /* See Table A-9                                              */
#define DEVICE_POWER_CYCLES                          0x0405
#define BURN_IN									  0x0440 /* Defined in ANSI E1.37-1                                    */

/* Category - Display Settings     */			      
#define DISPLAY_INVERT                               0x0500
#define DISPLAY_LEVEL                                0x0501

/* Category - Configuration        */
#define PAN_INVERT                                   0x0600
#define TILT_INVERT                                  0x0601
#define PAN_TILT_SWAP                                0x0602
#define REAL_TIME_CLOCK                              0x0603
#define LOCK_PIN                                   0x0640 /* Defined in ANSI E1.37-1                                    */
#define LOCK_STATE                                 0x0641 /* Defined in ANSI E1.37-1                                    */
#define LOCK_STATE_DESCRIPTION                     0x0642 /* Support required if MODULATION_FREQUENCY is supported      */

/* Category - Control              */
#define IDENTIFY_DEVICE                              0x1000
#define RESET_DEVICE                                 0x1001
#define POWER_STATE                                  0x1010 /* See Table A-11                                              */
#define PERFORM_SELFTEST                             0x1020 /* See Table A-10                                              */
#define SELF_TEST_DESCRIPTION                        0x1021
#define CAPTURE_PRESET                               0x1030
#define PRESET_PLAYBACK                              0x1031 /* See Table A-7                                               */
#define IDENTIFY_MODE                                0x1040 /* Defined in ANSI E1.37-1                                     */
#define PRESET_INFO                                  0x1041 /* Defined in ANSI E1.37-1                                     */
#define PRESET_STATUS                                0x1042 /* Defined in ANSI E1.37-1                                     */
#define PRESET_MERGEMODE                             0x1043 /* See E1.37-1 Table A-3                                       */
#define POWER_ON_SELF_TEST                           0x1044 /* Defined in ANSI E1.37-1                                     */


// NACK_REASON_CODES
#define NR_UNKNOWN_PID 0x0000
#define NR_FORMAT_ERROR 0x0001
#define NR_HARDWARE_FAULT 0x0002
#define NR_PROXY_REJECT 0x0003
#define NR_WRITE_PROTECT 0x0004
#define NR_UNSUPPORTED_COMMAND_CLASS 0x0005
#define NR_DATA_OUT_OF_RANGE 0x0006
#define NR_BUFFER_FULL 0x0007
#define NR_PACKET_SIZE_UNSUPPORTED 0x0008
#define NR_SUB_DEVICE_OUT_OF_RANGE 0x0009


// Response Types
#define RESPONSE_TYPE_ACK				 0x00
#define RESPONSE_TYPE_ACK_TIMER			 0x01
#define RESPONSE_TYPE_NACK_REASON		 0x02
#define RESPONSE_TYPE_ACK_OVERFLOW		 0x03
#define DISC_RESPONSE_LABEL				 0x19
#define RESPONSE_LABEL					 0x1D

extern "C" {

	/* Function : RDMInit
	* Purpose  : Initializes the lists/ strucutures to be used in the app.
	* Timeout: between 25 & 100
	* Return 0 on success
	**/
	uint16_t RDMInit (uint32_t ,uint32_t* ,uint32_t*, int ReadTimeout);

	/* Function : GetDeviceCount
	* Purpose  : retruns the number of devices found (in the list)
	*
	**/
	uint16_t GetDeviceCount();

	/* Function : GetDevice
	* Purpose  : Returns the wanted device serail (UID)
	*
	**/
	void GetDeviceID(uint8_t position, uint8_t * DeviceArray);

	/* Function : RDMDiscovery
	* Purpose  : Calls the Discovery routine
	*
	**/
	uint16_t RDMDiscovery(uint8_t Continous, volatile uint8_t* Ready, uint8_t* Message);

	/* Function : RDMTick
	* Purpose  : Calls the Timer Routine to Run every Millisec
	*
	**/
	void RDMTick();

	/* Function : RDMSet
	* Purpose  : Calls the RDM Set routine to act as a Set Request.
	*
	**/
	void RDMSet (uint8_t* SerialNum,uint16_t ParamID,uint8_t DataLength, void* Data, void *data_ptr, uint32_t* max_size,uint32_t Timeout,uint16_t Sub_Device,uint16_t* ResponseType);

	/* Function : RDMGet
	* Purpose  : Calls the RDM Get routine to act as a Get Request.
	*
	**/
	void RDMGet (uint8_t* SerialNum,uint16_t ParamID,uint8_t DataLength,void* Data ,void *data_ptr, uint32_t* max_size,uint32_t Timeout,uint16_t Sub_Device,uint16_t* ResponseType);

	/* Function : Get_Pro_Handle
	* Purpose  : Returns the USB PRO handle (acting as the controller)
	*
	**/
	void* Get_Pro_Handle();

	/* Function : SendDataToPro
	* Purpose  : Used to send DMX/Any  Data from the USB Pro acting as controller.
	* Use with label = 6 to send DMX     
	* Returns TRUE (1) on successfull Send , else 0
	**/
	uint8_t SendDataToPro(uint8_t label, uint8_t *data, uint32_t data_length);

	/* Function : ReceiveDataFromPro
	* Purpose  : Used to Receive Data from the USB Pro acting as controller.
	* Use with label = 5 to Recieve DMX      
	* Returns TRUE (1) on successfull Receive ,else 0
	**/
	uint8_t ReceiveDataFromPro(uint8_t label, uint8_t *data, uint32_t expected_length, uint32_t actual_length);

	/* Function : ClosePort
	* Purpose  : Closes the Port, removes all initialized data from memory.
	*
	**/
	void ClosePort();

	// BD Added since it was missing from header but documented in PDF

	void RDMComms(void);
}

// Valid Labels for use with DMX USB Pro Send Function
#define GET_WIDGET_PARAMS 3
#define GET_WIDGET_PARAMS_REPLY 3
#define SET_WIDGET_PARAMS 4
#define SET_DMX_RX_MODE 5
#define SET_DMX_TX_MODE 6
#define SEND_DMX_RDM_TX 7
#define RECEIVE_DMX_ON_CHANGE 8
#define RECEIVED_DMX_COS_TYPE 9
#define GET_WIDGET_SN 10

// Other useful defines for DMX USB PRO Comms
#define DMX_START_CODE 0x7E 
#define DMX_END_CODE 0xE7 
#define DMX_HEADER_LENGTH 4

